@@ -4,7 +4,7 @@
 ##  Plotting functions for 'solist', 'anylist', 'imlist'
 ##       and legacy class 'listof'
 ##
-##  $Revision: 1.32 $ $Date: 2022/05/24 03:51:42 $
+##  $Revision: 1.36 $ $Date: 2023/01/05 02:12:32 $
 ##
 
 plot.anylist <- plot.solist <- plot.listof <-
@@ -441,6 +441,7 @@ plot.anylist <- plot.solist <- plot.listof <-
     ## .... plot banner
     if(banner) {
       opa <- par(mar=rep.int(0,4), xpd=TRUE)
+      on.exit(par(opa))
       plot(numeric(0),numeric(0),type="n",ann=FALSE,axes=FALSE,
            xlim=c(-1,1),ylim=c(-1,1))
       cex <- resolve.1.default(list(cex.title=1.5), list(...))/par('cex')
@@ -448,7 +449,7 @@ plot.anylist <- plot.solist <- plot.listof <-
     }
     ## plot panels
     npa <- par(mar=mar.panel)
-    if(!banner) opa <- npa
+    if(!banner) on.exit(par(npa))
     result <- vector(mode="list", length=n)
     for(i in 1:n) {
       xi <- x[[i]]
@@ -477,7 +478,6 @@ plot.anylist <- plot.solist <- plot.listof <-
     }
     ## revert
     layout(1)
-    par(opa)
     return(invisible(result))
   }
 
@@ -530,11 +530,29 @@ plot.imlist <- local({
     stopifnot(is.list(ribargs))
     if(!is.null(ribsep))
       warning("Argument ribsep is not yet implemented for image arrays")
-    ## determine range of values
-    if(is.null(zlim))
-      zlim <- range(unlist(lapply(x, range)))
-    ## determine common colour map
-    imcolmap <- plot.im(x[[1L]], do.plot=FALSE, zlim=zlim, ..., ribn=ribn)
+    ## ascertain types of pixel values
+    xtypes <- sapply(x, getElement, name="type")
+    ischar <- (xtypes == "character")
+    if(any(ischar)) {
+      ## convert character-valued images to factor-valued
+      strings <- unique(unlist(lapply(x[ischar], "[")))
+      x[ischar] <- lapply(x[ischar], factorimage, levels=strings)
+      xtypes[ischar] <- "factor"
+    }
+    isfactor <- xtypes == "factor"
+    isnumeric <- xtypes %in% c("real", "integer", "logical")
+    if(all(isnumeric)) {
+      ## determine range of values for colour map
+      if(is.null(zlim))
+        zlim <- range(unlist(lapply(x, range)))
+      ## determine common colour map based on zlim
+      imcolmap <- plot.im(x[[1L]], do.plot=FALSE, zlim=zlim, ..., ribn=ribn)
+    } else if(all(isfactor)) {
+      x <- harmoniseLevels(x)
+      ## determine common colour map based on factor levels
+      imcolmap <- plot.im(x[[1L]], do.plot=FALSE, ..., ribn=ribn)
+    } else warning("Could not determine a common colour map for these images",
+                   call.=FALSE)
     ## plot ribbon?
     if(!ribbon) {
       ribadorn <- list()
@@ -564,8 +582,8 @@ plot.imlist <- local({
       ## bespoke function executed to plot colour ribbon
       do.ribbon <- function() {
         opa <- par(mar=ribmar)
+        on.exit(par(opa))
         do.call(plot, ribstuff)
-        par(opa)
       }
       ## ribbon plot function encoded as 'adorn' argument
       ribadorn <- list(adorn=do.ribbon, adorn.size=ribwid)
@@ -583,6 +601,10 @@ plot.imlist <- local({
     return(invisible(result))
   }
 
+  factorimage <- function(X, levels=NULL) {
+    eval.im(factor(X, levels=levels))
+  }
+  
   plot.imlist
 })
 
